#include "explosion.h"
#include "config.h"


/*
 *  Explosion
 */

Explosion::Explosion(Coord pos, Coord speed, XColor color, int duration)
: color_   (color)
, moves_   (0)
, duration_(duration)
{
	const int nb_particles (4 * duration * Details::Instance().Get());
	particles_.reserve(nb_particles);
	particles_speed_.reserve(nb_particles);
	for (int i (0); i < nb_particles; ++i) {
		using std::rand;
		const int rx (rand() % 16 + rand() % 16 - 16);
		const int ry (rand() % 16 + rand() % 16 - 16);
		const Coord particle (rx, ry);
		particles_.push_back(particle + pos);
		Coord particle_speed (particle / 4 + speed);
		particle_speed += Coord(rand() % 5 - 2, rand() % 5 - 2);
		particles_speed_.push_back(particle_speed);
	}
	XColor close_color = color_;
	X11::Inst().AllocColorAlways(&close_color);
	DrawAll(close_color.pixel);
}


void Explosion::Move()
{
	if (moves_ < duration_) {
		++moves_;
		DrawAll(X11::Inst().GetBlack());
		if (moves_ < duration_) {
			for (ParticlesCtr::size_type i (0);
			     i < particles_.size(); ++i) {
				particles_[i] += particles_speed_[i];
			}
			color_.red   = color_.red   * 98 / 100;
			color_.green = color_.green * 96 / 100;
			color_.blue  = color_.blue  * 96 / 100;
			if (color_.red < 48*1024 && color_.red > 36*1024) {
				color_.red   = color_.red   * 96 / 100;
				color_.green = color_.green * 92 / 100;
				color_.blue  = color_.blue  * 92 / 100;
			}
			XColor close_color = color_;
			X11::Inst().AllocColorAlways(&close_color);
			DrawAll(close_color.pixel);
		}
	}
}


void Explosion::DrawAll(X11::Color color) const
{
	X11::Inst().SetForeground(color);
	X11::Inst().SetClipMask(None);
	X11::Inst().DrawPoints(&particles_[0], particles_.size());
}
